/*
  Windows 2000 XP API Wrapper Pack
  Copyright (C) 2008 OldCigarette

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#define _WIN32_WINNT 0x0500
#define WINVER 0x0500

#include <windows.h>
#include <Wincrypt.h>
#include "../common/win2k_xp_debug.h"

typedef ULONG64 TRACEHANDLE, *PTRACEHANDLE;

/*Debugging*/
int debugLevel;
DbgPrintf_t DbgPrintf;
vDbgPrintf_t vDbgPrintf;

/*
CredDeleteW
CredEnumerateW
CredFree
CredReadW
CredWriteW
*/

typedef struct {
  ULONG i[2];
  ULONG buf[4];
  unsigned char in[64];
  unsigned char digest[16];
} MD5_CTX;

typedef void (WINAPI *MD5Init_t) (MD5_CTX* context);
typedef void (WINAPI *MD5Update_t) (MD5_CTX* context, const unsigned char* input, unsigned int inlen);
typedef void (WINAPI *MD5Final_t) (MD5_CTX* context);

MD5Init_t    _MD5Init;
MD5Update_t  _MD5Update;
MD5Final_t   _MD5Final;
HMODULE      CryptDll;
HCRYPTPROV   hCrypt;

HCRYPTPROV GetCryptProv() {
	if(!hCrypt) {
		if(!CryptAcquireContext(&hCrypt, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)) {
			if(GetLastError() == NTE_BAD_KEYSET) {
				CryptAcquireContext(&hCrypt, NULL, NULL, PROV_RSA_FULL, CRYPT_DELETEKEYSET);
				if(!CryptAcquireContext(&hCrypt, NULL, NULL, PROV_RSA_FULL, CRYPT_NEWKEYSET|CRYPT_VERIFYCONTEXT)) {
					if(DbgPrintf) DbgPrintf(DBG_ERROR, S_RED "advapi32: Failed to acquire or create crypt provider 0x%08X\n", GetLastError());
				}
			}
		}
	}
	return hCrypt;
}

HMODULE GetCryptDll() {
	if(!CryptDll) {
		CryptDll = LoadLibraryA("cryptdll.dll");
		if(!CryptDll && DbgPrintf) DbgPrintf(DBG_ERROR, S_RED "advapi32: Failed to load cryptdll.dll 0x%08X\n", GetLastError());
	}
	return CryptDll;
}

void WINAPI MD5Init(MD5_CTX* context) {
	if(!_MD5Init) _MD5Init = GetProcAddress(GetCryptDll(), "MD5Init");
	_MD5Init(context);
}

void WINAPI MD5Update(MD5_CTX* context, const unsigned char* input, unsigned int inlen) {
	if(!_MD5Update) _MD5Update = GetProcAddress(GetCryptDll(), "MD5Update");
	_MD5Update(context, input, inlen);
}

void WINAPI MD5Final(MD5_CTX* context) {
	if(!_MD5Final) _MD5Final = GetProcAddress(GetCryptDll(), "MD5Final");
	_MD5Final(context);
}

BOOL WINAPI SystemFunction036(PVOID pbBuffer, ULONG dwLen) {
	return CryptGenRandom(GetCryptProv(), dwLen, pbBuffer);
}

ULONG WINAPI TraceMessage(TRACEHANDLE SessionHandle, ULONG MessageFlags, LPGUID MessageGuid,
  USHORT MessageNumber, ...) {
	if(DbgPrintf)
		DbgPrintf(DBG_WARN, S_YELLOW "advapi32: TraceMessage MsgNum=0x%04X MsgFlags=0x%08X\n",
			MessageNumber, MessageFlags);
    return ERROR_SUCCESS;
}

ULONG WINAPI TraceMessageVa(TRACEHANDLE SessionHandle, ULONG MessageFlags, LPGUID MessageGuid,
  USHORT MessageNumber, va_list MessageArgList) {
	if(DbgPrintf)
		DbgPrintf(DBG_WARN, S_YELLOW "advapi32: TraceMessageVa MsgNum=0x%04X MsgFlags=0x%08X\n",
			MessageNumber, MessageFlags);
    return ERROR_SUCCESS;
}

DWORD WINAPI DelayedSystemShutdown(LPVOID lpParameter) {
	DWORD *params = (DWORD *)lpParameter;
	
	Sleep(params[0] * 1000);
	ExitWindowsEx(params[1], 0);
	
	free(params);
	return 0;
}

BOOL WINAPI SystemShutdownWrapper(DWORD timeout, BOOL force, BOOL reboot) {
	DWORD flags = 0;
	DWORD *params;
	
	if(force)  flags |= EWX_FORCEIFHUNG;
	if(reboot) flags |= EWX_REBOOT;
	else       flags |= EWX_POWEROFF;
	
	if(timeout == 0) {
		return ExitWindowsEx(flags, 0);
	}
	
	params = (DWORD *)malloc(sizeof(DWORD)*2);
	params[0] = timeout;
	params[1] = flags;
	
	return CreateThread(NULL, 0, DelayedSystemShutdown, params, 0, NULL) != NULL;
}

BOOL WINAPI XP_InitiateSystemShutdownExA(LPSTR lpMachineName, LPSTR lpMessage, DWORD dwTimeout, BOOL bForceAppsClosed, BOOL bRebootAfterShutdown, DWORD dwReason) {
	if(lpMachineName) return InitiateSystemShutdownExA(lpMachineName, lpMessage, dwTimeout, bForceAppsClosed, bRebootAfterShutdown, dwReason);
	else              return SystemShutdownWrapper(dwTimeout, bForceAppsClosed, bRebootAfterShutdown);
}
BOOL WINAPI XP_InitiateSystemShutdownExW(LPWSTR lpMachineName, LPWSTR lpMessage, DWORD dwTimeout, BOOL bForceAppsClosed, BOOL bRebootAfterShutdown, DWORD dwReason) {
	if(lpMachineName) return InitiateSystemShutdownExW(lpMachineName, lpMessage, dwTimeout, bForceAppsClosed, bRebootAfterShutdown, dwReason);
	else              return SystemShutdownWrapper(dwTimeout, bForceAppsClosed, bRebootAfterShutdown);
}
BOOL WINAPI XP_InitiateSystemShutdownA(LPSTR lpMachineName, LPSTR lpMessage, DWORD dwTimeout, BOOL bForceAppsClosed, BOOL bRebootAfterShutdown) {
	if(lpMachineName) return InitiateSystemShutdownA(lpMachineName, lpMessage, dwTimeout, bForceAppsClosed, bRebootAfterShutdown);
	else              return SystemShutdownWrapper(dwTimeout, bForceAppsClosed, bRebootAfterShutdown);
}
BOOL WINAPI XP_InitiateSystemShutdownW(LPWSTR lpMachineName, LPWSTR lpMessage, DWORD dwTimeout, BOOL bForceAppsClosed, BOOL bRebootAfterShutdown) {
	if(lpMachineName) return InitiateSystemShutdownW(lpMachineName, lpMessage, dwTimeout, bForceAppsClosed, bRebootAfterShutdown);
	else              return SystemShutdownWrapper(dwTimeout, bForceAppsClosed, bRebootAfterShutdown);
}

ULONG WINAPI XP_QueryTraceA(TRACEHANDLE SessionHandle, PSTR SessionName, PVOID Properties) {
	if(DbgPrintf) DbgPrintf(DBG_WARN, S_YELLOW "advapi32: QueryTraceA stub %s\n", SessionName);
	return ERROR_ACCESS_DENIED;
}

ULONG WINAPI XP_QueryTraceW(TRACEHANDLE SessionHandle, PWSTR SessionName, PVOID Properties) {
	if(DbgPrintf) DbgPrintf(DBG_WARN, S_YELLOW "advapi32: QueryTraceA stub %s\n", SessionName);
	return ERROR_ACCESS_DENIED;
}

LONG WINAPI XP_RegOpenKeyExA(HKEY hKey, LPCSTR lpSubKey, DWORD ulOptions, REGSAM samDesired, PHKEY phkResult)
{
	return RegOpenKeyExA(hKey, lpSubKey, ulOptions, samDesired & ~(KEY_WOW64_32KEY|KEY_WOW64_64KEY), phkResult);
}

LONG WINAPI XP_RegOpenKeyExW(HKEY hKey, LPCWSTR lpSubKey, DWORD ulOptions, REGSAM samDesired, PHKEY phkResult)
{
	return RegOpenKeyExW(hKey, lpSubKey, ulOptions, samDesired & ~(KEY_WOW64_32KEY|KEY_WOW64_64KEY), phkResult);
}

BOOL WINAPI DllMain(
  HINSTANCE hinstDLL,
  DWORD fdwReason,
  LPVOID lpvReserved) {
	DebugLevel_t DebugLevel;
	
  	switch(fdwReason) {
		case DLL_PROCESS_ATTACH:
			//Init debug data
			DbgPrintf = GetDbgPrintf();
			vDbgPrintf = GetvDbgPrintf();
			DebugLevel = GetDebugLevel();
			
			if(DebugLevel) debugLevel = DebugLevel();
			else           debugLevel = 0;
			
			if(DbgPrintf) DbgPrintf(DBG_ALWAYS, "advapi32: ADVAPI32.DLL Wrapper Init\n");
			break;
		case DLL_PROCESS_DETACH:
			if(hCrypt) CryptReleaseContext(hCrypt,0);
	}
	
	return TRUE;
}

