/*
  Windows 2000 XP API Wrapper Pack
  Copyright (C) 2008 OldCigarette

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#define _WIN32_IE 0x0600

#include <windows.h>
#include <shlobj.h>
#include <shobjidl.h>
#include <shlwapi.h>
#include "../common/win2k_xp_config.h"

/*Debugging*/
#include "../common/win2k_xp_debug.h"
int debugLevel;
DbgPrintf_t DbgPrintf;

typedef BOOL (WINAPI *FileIconInit_t) (BOOL fRestoreCache);
typedef BOOL (WINAPI *Shell_GetImageLists_t) (HIMAGELIST *phiml, HIMAGELIST *phimlSmall);

FileIconInit_t FileIconInit;
Shell_GetImageLists_t _Shell_GetImageLists;

HRESULT WINAPI XP_SHGetFolderPathW(HWND hwndOwner, int nFolder, HANDLE hToken, DWORD dwFlags, LPWSTR pszPath);

HRESULT WINAPI XP_SHOpenFolderAndSelectItems(LPCITEMIDLIST pidlFolder, UINT cidl, LPCITEMIDLIST *apidl, DWORD dwFlags) {
	if(DbgPrintf) DbgPrintf(DBG_WARN, S_YELLOW "shell32: SHOpenFolderAndSelectItems stub\n");
	return S_OK;
}


//The value returned by this should be ok to use with the ImageList_ APIs BUT
//I don't think if you try to use the COM interface it's going to work very well
HRESULT WINAPI XP_SHGetImageList(int iImageList, REFIID riid, void **ppv) {
	HIMAGELIST s, l;
	if(!_Shell_GetImageLists(&l, &s)) {
		if(DbgPrintf) DbgPrintf(DBG_WARN, S_YELLOW "shell32: GetImageLists failed\n");
		return E_FAIL;
	}
	switch(iImageList) {
		case SHIL_SMALL:
		case SHIL_SYSSMALL:
			*ppv = s;
			return S_OK;
		case SHIL_LARGE:
		case SHIL_EXTRALARGE:
		//case SHIL_JUMBO:
			*ppv = l;
			return S_OK;
		default:
			return E_FAIL; 
	}
}

HRESULT WINAPI XP_SHParseDisplayName(
	LPCWSTR pszName, IBindCtx *pbc, LPITEMIDLIST *ppidl, SFGAOF sfgaoIn, SFGAOF *psfgaoOut) {
	
	IShellFolder *shellFolder;
	HRESULT result;
	ULONG attrib = sfgaoIn;
	
	result = SHGetDesktopFolder(&shellFolder);
	if(result == S_OK) {
		result = shellFolder->lpVtbl->ParseDisplayName(shellFolder, NULL, pbc, pszName, NULL, ppidl, &attrib);
		if(result == S_OK && psfgaoOut) *psfgaoOut = attrib;
	}
	
	return result;
}

HRESULT WINAPI XP_SHGetFolderPathAndSubDirW(HWND hwnd, int csidl, HANDLE hToken,
  DWORD dwFlags, LPCWSTR pszSubDir, LPWSTR pszPath) {
	HRESULT hr;
	int err;
	BOOL create;
	
	hr = XP_SHGetFolderPathW(hwnd, csidl, hToken, dwFlags, pszPath);
	if(hr != S_OK) {
		pszPath[0] = L'\0';
		return hr;
	}
	
	create = CSIDL_FLAG_CREATE & csidl;

	if(!PathAppendW(pszPath, pszSubDir)) {
		pszPath[0] = L'\0';
		return S_FALSE;
	}
	
	if(create) {
		err = SHCreateDirectory(hwnd, pszPath);
		if(err != ERROR_SUCCESS &&
		   err != ERROR_FILE_EXISTS &&
		   err != ERROR_ALREADY_EXISTS) {
			pszPath[0] = L'\0';
			return S_FALSE;
		}
	}
	
	return S_OK;
}

HRESULT WINAPI XP_SHGetFolderPathAndSubDirA(HWND hwnd, int csidl, HANDLE hToken,
  DWORD dwFlags, LPCSTR pszSubDir, LPSTR pszPath) {
	WCHAR path  [MAX_PATH];
	WCHAR subdir[MAX_PATH];
	HRESULT hr;
	
	if(MultiByteToWideChar(CP_ACP, 0, pszSubDir, -1, subdir, ARRAY_SIZE(subdir)) == 0) {
		pszPath = '\0';
		return S_FALSE;
	}
	
	hr = XP_SHGetFolderPathAndSubDirW(hwnd, csidl, hToken, dwFlags, subdir, path);
	
	if(hr == S_OK) {
		WideCharToMultiByte(CP_ACP, 0, path, -1, pszPath, MAX_PATH, NULL, NULL);
	} else {
		pszPath = '\0';
	}
	
	return hr;
}

HRESULT WINAPI XP_SHGetFolderPathA(HWND hwndOwner, int nFolder, HANDLE hToken, DWORD dwFlags, LPSTR pszPath) {
	if((nFolder & ~CSIDL_FLAG_CREATE) == CSIDL_MYMUSIC) {
		return XP_SHGetFolderPathAndSubDirA(hwndOwner, (nFolder & CSIDL_FLAG_CREATE) | CSIDL_PERSONAL, hToken, dwFlags, "My Music", pszPath);
	} else if((nFolder & ~CSIDL_FLAG_CREATE) == CSIDL_MYVIDEO) {
		return XP_SHGetFolderPathAndSubDirA(hwndOwner, (nFolder & CSIDL_FLAG_CREATE) | CSIDL_PERSONAL, hToken, dwFlags, "My Videos", pszPath);
	} else if((nFolder & ~CSIDL_FLAG_CREATE) == CSIDL_MYDOCUMENTS) {
		return SHGetFolderPathA(hwndOwner, (nFolder & CSIDL_FLAG_CREATE) | CSIDL_PERSONAL, hToken, dwFlags, pszPath);
	} else {
		return SHGetFolderPathA(hwndOwner, nFolder, hToken, dwFlags, pszPath);
	}
}

HRESULT WINAPI XP_SHGetFolderPathW(HWND hwndOwner, int nFolder, HANDLE hToken, DWORD dwFlags, LPWSTR pszPath) {
	if((nFolder & ~CSIDL_FLAG_CREATE) == CSIDL_MYMUSIC) {
		return XP_SHGetFolderPathAndSubDirW(hwndOwner, (nFolder & CSIDL_FLAG_CREATE) | CSIDL_PERSONAL, hToken, dwFlags, L"My Music", pszPath);
	} else if((nFolder & ~CSIDL_FLAG_CREATE) == CSIDL_MYVIDEO) {
		return XP_SHGetFolderPathAndSubDirW(hwndOwner, (nFolder & CSIDL_FLAG_CREATE) | CSIDL_PERSONAL, hToken, dwFlags, L"My Videos", pszPath);
	} else if((nFolder & ~CSIDL_FLAG_CREATE) == CSIDL_MYDOCUMENTS) {
		return SHGetFolderPathW(hwndOwner, (nFolder & CSIDL_FLAG_CREATE) | CSIDL_PERSONAL, hToken, dwFlags, pszPath);
	} else {
		return SHGetFolderPathW(hwndOwner, nFolder, hToken, dwFlags, pszPath);
	}
}

HRESULT WINAPI XP_SHCreateFileExtractIconW(LPCWSTR pszFile, DWORD dwFileAttributes,
  REFIID riid, void **ppv) {
	IShellFolder *shellFolder;
	LPITEMIDLIST pidl;
	HRESULT result;
	
	result = SHGetDesktopFolder(&shellFolder);
	if(result == S_OK) {
		result = shellFolder->lpVtbl->ParseDisplayName(shellFolder, NULL, NULL, pszFile, NULL, &pidl, NULL);
		if(result == S_OK) {
			result = shellFolder->lpVtbl->GetUIObjectOf(shellFolder, NULL, 1, &pidl, riid, NULL, ppv);
		}
	}
	
	return result;
}

BOOL WINAPI DllMain(
  HINSTANCE hinstDLL,
  DWORD fdwReason,
  LPVOID lpvReserved) {
	DebugLevel_t DebugLevel;
	HMODULE hOrg;
	
  	switch(fdwReason) {
		case DLL_PROCESS_ATTACH:
			//Init debug data
			DbgPrintf = GetDbgPrintf();
			DebugLevel = GetDebugLevel();
			
			if(DebugLevel) debugLevel = DebugLevel();
			else           debugLevel = 0;
			
			if(DbgPrintf) DbgPrintf(DBG_ALWAYS, "shell32: SHELL32.DLL Wrapper Init\n");
			hOrg = GetModuleHandleA("SHELL32_ORG.DLL");
			
			if(hOrg == NULL) {
				if(DbgPrintf) DbgPrintf(DBG_WARN, S_YELLOW "shell32: couldn't find SHELL32_ORG things may break!\n");
			} else {
				FileIconInit = (FileIconInit_t)GetProcAddress(hOrg, 660);
				if(!FileIconInit || !FileIconInit(TRUE)) {
					if(DbgPrintf) DbgPrintf(DBG_WARN, S_YELLOW "shell32: couldn't init icon cache\n");
				}
				_Shell_GetImageLists = (Shell_GetImageLists_t)GetProcAddress(hOrg, 71);
				if(!_Shell_GetImageLists) {
					if(DbgPrintf) DbgPrintf(DBG_WARN, S_YELLOW "shell32: couldn't get Shell_GetImageLists\n");
				}
			}
			break;
	}
	
	return TRUE;
}
