/*
  Windows 2000 XP API Wrapper Pack
  Copyright (C) 2008 OldCigarette

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <windows.h>

#define E_PROP_ID_UNSUPPORTED            ((HRESULT)0x80070490)


/*Debugging*/
#include "../common/win2k_xp_debug.h"
int debugLevel;
DbgPrintf_t DbgPrintf;

typedef HANDLE HTHEME;
typedef enum {
    TS_MIN,
    TS_TRUE,
    TS_DRAW
} THEME_SIZE;


DWORD GetStringW(DWORD len, LPWSTR dest, LPCWSTR source) {
	DWORD c;
	PCWSTR p;

	for(c = 0, p = source; c < len && *p != L'\0'; c++, p++) {
		dest[c] = *p;
	}

	if(c < len && *p == L'\0') {
		dest[c] = L'\0';
		return c;
	}

	while(*p) { c++; p++; }
	return c + 1;
}

HRESULT WINAPI XP_GetCurrentThemeName(      
    LPWSTR pszThemeFileName,
    int dwMaxNameChars,
    LPWSTR pszColorBuff,
    int cchMaxColorChars,
    LPWSTR pszSizeBuff,
    int cchMaxSizeChars
) {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: GetCurrentThemeName not implemented\n");
	
	if(pszThemeFileName) {
		GetStringW(dwMaxNameChars, pszThemeFileName, L"win2k_theme");
	}
	if(pszColorBuff) {
		GetStringW(cchMaxColorChars, pszColorBuff, L"win2k_colors");
	}
	if(pszSizeBuff) {
		GetStringW(cchMaxSizeChars, pszSizeBuff, L"win2k_size");
	}
	return S_OK;
}

BOOL WINAPI XP_IsAppThemed(VOID) {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: IsAppThemed returning false\n");
	return FALSE;
}

/*
return NULL, otherwise firefox will die/look like shit
*/
HTHEME WINAPI XP_OpenThemeData(HWND hwnd, LPCWSTR pszClassList) {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: OpenThemeData not implemented\n");
	return NULL;
}

HRESULT WINAPI XP_CloseThemeData(HTHEME hTheme) {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: CloseThemeData not implemented\n");
	return S_OK;
}

HRESULT WINAPI XP_DrawThemeBackground(      
    HTHEME hTheme,
    HDC hdc,
    int iPartId,
    int iStateId,
    const RECT *pRect,
    const RECT *pClipRect) {
	HBRUSH b;
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: DrawThemeBackground not implemented\n");
	b = CreateSolidBrush(RGB(0xFF,0,0xFF));
	FillRect(hdc, pRect, b);
	DeleteObject(b);
	return S_FALSE;
}

BOOL WINAPI XP_IsThemeBackgroundPartiallyTransparent(      
  HTHEME hTheme,
  int iPartId,
  int iStateId) {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: IsThemeBackgroundPartiallyTransparent not implemented\n");
	return FALSE; //no transparency on 2k
}

HRESULT WINAPI XP_DrawThemeParentBackground(HWND hwnd, HDC hdc, const RECT *prc) {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: DrawThemeParentBackground stub\n");
	return S_FALSE;
}

HRESULT WINAPI XP_DrawThemeText(      
    HTHEME hTheme,
    HDC hdc,
    int iPartId,
    int iStateId,
    LPCWSTR pszText,
    int iCharCount,
    DWORD dwTextFlags,
    DWORD dwTextFlags2,
    LPCRECT pRect) {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: DrawThemeText stub '%S'\n", pszText);
	return DrawTextW(hdc, pszText, iCharCount, pRect, dwTextFlags) ? S_OK : S_FALSE;
}

HRESULT WINAPI XP_GetThemeBackgroundContentRect(      
    HTHEME hTheme,
    HDC hdc,
    int iPartId,
    int iStateId,
    LPCRECT pBoundingRect,
    LPRECT pContentRect) {
	memcpy(pContentRect, pBoundingRect, sizeof(RECT));
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: GetThemeBackgroundContentRect stub\n");
	return S_FALSE;
}

HRESULT WINAPI XP_GetThemeColor(      
	HTHEME hTheme,
	int iPartId,
	int iStateId,
	int iPropId,
	COLORREF *pColor) {
	*pColor = 0x00FFFFFF;
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: GetThemeColor stub, returning white\n");
	return S_OK;
}

COLORREF WINAPI XP_GetThemeSysColor(HTHEME hTheme, int iColorID) {
	return GetSysColor(iColorID);
}

BOOL WINAPI XP_IsThemeActive() {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: IsThemeActive stub, returning FALSE\n");
	return FALSE;
}

BOOL WINAPI XP_IsThemeDialogTextureEnabled(HWND hwnd) {
	return FALSE;
}

BOOL WINAPI XP_IsThemePartDefined(HTHEME hTheme, int iPartId, int iStateId) {
	return FALSE;
}

int CALLBACK GetFirstLogFont(const LOGFONT *lplf, const TEXTMETRIC *lptm, DWORD dwType, LPARAM lpData) {
	memcpy((PVOID)lpData, lplf, sizeof(LOGFONT));
	return 0;
}

HRESULT WINAPI XP_GetThemeFont(
    HTHEME hTheme,
    HDC hdc,
    int iPartId,
    int iStateId,
    int iPropId,
    LOGFONT *pFont) {
	if(hdc != NULL) {
		if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: GetThemeFont stub, returning first EnumFonts\n");
		EnumFonts(hdc, NULL, GetFirstLogFont, (LPARAM)pFont);
		return S_OK;
	} else {
		if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: GetThemeFont stub, returning E_PROP_ID_UNSUPPORTED\n");
		return E_PROP_ID_UNSUPPORTED;
	}
}

HRESULT WINAPI XP_GetThemeTextMetrics(      
    HTHEME hTheme,
    HDC hdc,
    int iPartId,
    int iStateId,
    TEXTMETRIC *ptm) {
	HRESULT r;
	if(hdc != NULL) {
		if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: GetThemeTextMetrics stub, returning GetTextMetrics\n");
		return GetTextMetrics(hdc, ptm) ? S_OK : S_FALSE;
	} else {
		if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: GetThemeTextMetrics stub, returning S_FALSE\n");
		return S_FALSE;
	}
}

HRESULT WINAPI XP_GetThemePartSize(      
    HTHEME hTheme,
    HDC hdc,
    int iPartId,
    int iStateId,
    LPCRECT prc,
    THEME_SIZE eSize,
    SIZE *psz) {
	psz->cx = psz->cy = 1;
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: GetThemePartSize stub, returning S_FALSE\n");
	return S_FALSE;
}

HRESULT WINAPI XP_EnableThemeDialogTexture(HWND hwnd, DWORD dwFlags) {
	if(DbgPrintf) DbgPrintf(DBG_INFO, "uxtheme: EnableThemeDialogTexture stub, returning S_OK\n");
	return S_FALSE;
}

BOOL WINAPI DllMain(
  HINSTANCE hinstDLL,
  DWORD fdwReason,
  LPVOID lpvReserved) {
	DebugLevel_t DebugLevel;
	
	switch(fdwReason) {
		case DLL_PROCESS_ATTACH:
			//Init debug data
			DbgPrintf = GetDbgPrintf();
			DebugLevel = GetDebugLevel();
			
			if(DebugLevel) debugLevel = DebugLevel();
			else           debugLevel = 0;
			
			if(DbgPrintf) DbgPrintf(DBG_ALWAYS, "uxtheme: UXTHEME.DLL Wrapper Init\n");
			break;
	}
	
	return TRUE;
}
